@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -26,7 +26,7 @@ import logging
 
 LOGGER = logging.getLogger( 'camelot.view.controls.editors.onetomanyeditor' )
 
-from PyQt4 import QtGui
+from PyQt4 import QtGui, QtCore
 from PyQt4.QtCore import Qt
 from wideeditor import WideEditor
 from customeditor import CustomEditor
@@ -39,7 +39,10 @@ from camelot.view import register
 class One2ManyEditor(CustomEditor, WideEditor):
 
     new_icon = Icon('tango/16x16/actions/document-new.png')
-    
+    delete_icon = Icon( 'tango/16x16/places/user-trash.png' )
+    copy_icon = Icon( 'tango/16x16/actions/edit-copy.png' )
+    spreadsheet_icon = Icon( 'tango/16x16/mimetypes/x-office-spreadsheet.png' )
+
     def __init__( self,
                  admin = None,
                  parent = None,
@@ -65,17 +68,17 @@ class One2ManyEditor(CustomEditor, WideEditor):
         #
         # Setup table
         #
-        from camelot.view.controls.tableview import TableWidget
+        from camelot.view.controls.tableview import AdminTableWidget
         # parent set by layout manager
-        self.table = TableWidget()
+        table = AdminTableWidget(admin, self)
+        table.setObjectName('table')
         rowHeight = QtGui.QFontMetrics( self.font() ).height() + 5
-        self.table.verticalHeader().setDefaultSectionSize( rowHeight )
         layout.setSizeConstraint( QtGui.QLayout.SetNoConstraint )
         self.setSizePolicy( QtGui.QSizePolicy.Expanding,
-                           QtGui.QSizePolicy.Expanding )
+                            QtGui.QSizePolicy.Expanding )
         self.setMinimumHeight( rowHeight*5 )
         if vertical_header_clickable:
-            self.table.verticalHeader().sectionClicked.connect(
+            table.verticalHeader().sectionClicked.connect(
                 self.createFormForIndex
             )
         self.admin = admin
@@ -83,8 +86,8 @@ class One2ManyEditor(CustomEditor, WideEditor):
         self.add_button = None
         self.copy_button = None
         self.delete_button = None
-        layout.addWidget( self.table )
-        self.setupButtons( layout )
+        layout.addWidget( table )
+        self.setupButtons( layout, table )
         self.setLayout( layout )
         self.model = None
         self._new_message = None
@@ -95,15 +98,14 @@ class One2ManyEditor(CustomEditor, WideEditor):
         self.delete_button.setEnabled(editable)
         self._new_message = new_message
 
-    def setupButtons( self, layout ):
+    def setupButtons( self, layout, table ):
         button_layout = QtGui.QVBoxLayout()
         button_layout.setSpacing( 0 )
         self.delete_button = QtGui.QToolButton()
-        icon = Icon( 'tango/16x16/places/user-trash.png' ).getQIcon()
-        self.delete_button.setIcon( icon )
+        self.delete_button.setIcon( self.delete_icon.getQIcon() )
         self.delete_button.setAutoRaise( True )
         self.delete_button.setToolTip(_('Delete'))
-        self.delete_button.clicked.connect(self.deleteSelectedRows)
+        self.delete_button.clicked.connect( table.delete_selected_rows )
         self.add_button = QtGui.QToolButton()
         icon = self.new_icon.getQIcon()
         self.add_button.setIcon( icon )
@@ -111,13 +113,12 @@ class One2ManyEditor(CustomEditor, WideEditor):
         self.add_button.setToolTip(_('New'))
         self.add_button.clicked.connect(self.newRow)
         self.copy_button = QtGui.QToolButton()
-        icon = Icon( 'tango/16x16/actions/edit-copy.png' ).getQIcon()
-        self.copy_button.setIcon( icon )
+        self.copy_button.setIcon( self.copy_icon.getQIcon() )
         self.copy_button.setAutoRaise( True )
         self.copy_button.setToolTip(_('Copy'))
-        self.copy_button.clicked.connect(self.copy_selected_rows)
+        self.copy_button.clicked.connect( table.copy_selected_rows )
         export_button = QtGui.QToolButton()
-        export_button.setIcon( Icon( 'tango/16x16/mimetypes/x-office-spreadsheet.png' ).getQIcon() )
+        export_button.setIcon( self.spreadsheet_icon.getQIcon() )
         export_button.setAutoRaise( True )
         export_button.setToolTip(_('Export as spreadsheet'))
         export_button.clicked.connect(self.exportToExcel)
@@ -135,83 +136,82 @@ class One2ManyEditor(CustomEditor, WideEditor):
             title = self.admin.get_verbose_name_plural()
             columns = self.admin.get_columns()
             if self.model:
-                data = list( self.model.getData() )
-                open_data_with_excel( title, columns, data )
+                data = list(self.model.getData())
+                for i, row in enumerate(data):
+                    for j, column in enumerate(row):
+                        if isinstance(column, basestring):
+                            row[j] = _(column)
+                    data[i] = row
+                open_data_with_excel(title, columns, data)
 
         post( export )
 
     def getModel( self ):
         return self.model
 
+    @QtCore.pyqtSlot(object)
     def update_delegates( self, *args ):
-        if self.model:
+        table = self.findChild(QtGui.QWidget, 'table')
+        if self.model and table:
             delegate = self.model.getItemDelegate()
             if delegate:
-                self.table.setItemDelegate( delegate )
+                table.setItemDelegate( delegate )
                 for i in range( self.model.columnCount() ):
                     txtwidth = self.model.headerData( i, Qt.Horizontal, Qt.SizeHintRole ).toSize().width()
-                    colwidth = self.table.columnWidth( i )
-                    self.table.setColumnWidth( i, max( txtwidth, colwidth ) )
+                    colwidth = table.columnWidth( i )
+                    table.setColumnWidth( i, max( txtwidth, colwidth ) )
 
     def set_value( self, model ):
         model = CustomEditor.set_value( self, model )
-        if model and model != self.model:
+        table = self.findChild(QtGui.QWidget, 'table')
+        if table and model and model != self.model:
             self.model = model
-            self.table.setModel( model )
-            register.register( self.model, self.table )
+            table.setModel( model )
+            register.register( self.model, table )
             post( model._extend_cache, self.update_delegates )
 
     @gui_function
-    def activate_editor( self, row ):
-        index = self.model.index( row, 0 )
-        self.table.scrollToBottom()
-        self.table.setCurrentIndex( index )
-        self.table.edit( index )
+    def activate_editor( self, number_of_rows ):
+#        return
+# Activating this code can cause segfaults
+# see ticket 765 in web issues
+#
+# The segfault seems no longer there after disabling the
+# editor before setting a new model, but the code below
+# seems to have no effect.
+        table = self.findChild(QtGui.QWidget, 'table')
+        if table:
+            index = self.model.index( max(0, number_of_rows-1), 0 )
+            table.scrollToBottom()
+            table.setCurrentIndex( index )
+            table.edit( index )
 
     def newRow( self ):
         from camelot.view.workspace import show_top_level
         if self._new_message:
             QtGui.QMessageBox.information(self, _('New'), self._new_message)
             return
-        
+
         if self.create_inline:
 
             @model_function
             def create():
-                o = self.admin.entity()
-                row = self.model.insertEntityInstance( 0, o )
-                return row
+                return self.model.append_object( self.admin.entity() )
 
             post( create, self.activate_editor )
 
         else:
-            prependentity = lambda o: self.model.insertEntityInstance( 0, o )
-            removeentity = lambda o: self.model.removeEntityInstance( o )
-            form = self.admin.create_new_view( parent = None,
-                                               oncreate = prependentity,
-                                               onexpunge = removeentity )
+            form = self.admin.create_new_view( related_collection_proxy=self.model, parent = None )
             show_top_level( form, self )
 
-    def copy_selected_rows( self ):
-        """Copy the selected rows in this tableview"""
-        LOGGER.debug( 'delete selected rows called' )
-        for row in set( map( lambda x: x.row(), self.table.selectedIndexes() ) ):
-            self.model.copy_row( row )
-
-    def deleteSelectedRows( self ):
-        """Delete the selected rows in this tableview"""
-        LOGGER.debug( 'delete selected rows called' )
-        self.model.remove_rows( set( map( lambda x: x.row(), self.table.selectedIndexes() ) ) )
-
     def createFormForIndex( self, index ):
         from camelot.view.workspace import show_top_level
         from camelot.view.proxy.collection_proxy import CollectionProxy
         model = CollectionProxy( self.admin,
-                                 self.model.collection_getter,
+                                 self.model.get_collection,
                                  self.admin.get_fields,
                                  max_number_of_rows = 1,
                                  edits = None )
         form = self.admin.create_form_view( u'', model, self.model.map_to_source(index) )
         show_top_level( form, self )
 
-
