@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -43,6 +43,7 @@ from camelot.view.utils import (
     string_from_string,
     enumeration_to_string,
     default_language,
+    code_from_string,
 )
 
 _numerical_operators = (operator.eq, operator.ne, operator.lt, operator.le, operator.gt, operator.ge, between_op)
@@ -106,7 +107,7 @@ _sqlalchemy_to_python_type_ = {
 
     sqlalchemy.types.Float: lambda f: {
         'python_type': float,
-        'precision': f.precision if not isinstance(f.precision, tuple) else f.precision[1],
+        'precision': (f.precision if not isinstance(f.precision, tuple) else f.precision[1]) or 2,
         'editable': True,
         'minimum': constants.camelot_minfloat,
         'maximum': constants.camelot_maxfloat,
@@ -115,7 +116,7 @@ _sqlalchemy_to_python_type_ = {
         'from_string': float_from_string,
         'operators': _numerical_operators,
     },
-    
+
     sqlalchemy.types.Numeric: lambda f: {
         'python_type': float,
         'precision': f.scale,
@@ -206,6 +207,7 @@ _sqlalchemy_to_python_type_ = {
         'parts': f.parts,
         'separator': f.separator,
         'operators' : _text_operators,
+        'from_string' : lambda s:code_from_string(s, f.separator),
     },
 
     camelot.types.IPAddress: lambda f: {
@@ -266,7 +268,7 @@ _sqlalchemy_to_python_type_ = {
         'widget': 'combobox',
         'operators' : _numerical_operators,
     },
-    
+
     camelot.types.Language: lambda f: {
         'delegate': delegates.LanguageDelegate,
         'python_type': str,
@@ -275,7 +277,7 @@ _sqlalchemy_to_python_type_ = {
         'editable': True,
         'nullable': False,
         'widget': 'combobox',
-    },    
+    },
 
     camelot.types.File : lambda f: {
         'python_type': str,
@@ -283,6 +285,7 @@ _sqlalchemy_to_python_type_ = {
         'delegate': delegates.FileDelegate,
         'storage': f.storage,
         'operators' : _text_operators,
+        'remove_original': False,
     },
 }
 
@@ -316,8 +319,8 @@ field_types.sort(lambda x, y: cmp(x.__name__, y.__name__))
 for field_type in field_types:
     field_attributes = _sqlalchemy_to_python_type_[field_type](DummyField())
     delegate = field_attributes['delegate']
-    row = row_format%(field_type.__name__, 
-                      ':ref:`%s <delegate-%s>`'%(delegate.__name__, delegate.__name__), 
+    row = row_format%(field_type.__name__,
+                      ':ref:`%s <delegate-%s>`'%(delegate.__name__, delegate.__name__),
                       '.. image:: ../_static/editors/%s_editable.png'%(delegate.editor.__name__))
     doc += row + """
 """ + row_separator + """
@@ -328,3 +331,4 @@ doc += """
 
 __doc__ = doc
 
+
