@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -21,10 +21,14 @@
 #  project-camelot@conceptive.be
 #
 #  ============================================================================
+
 """Main function, to be called to start the GUI interface"""
 from PyQt4 import QtCore
 from camelot.core.utils import ugettext as _
 
+from camelot.art import resources # Required for tooltip visualization
+resources.__name__ # Dodge PyFlakes' attack
+
 class Application(QtCore.QObject):
     """The camelot application.  This class will take care of the order of
     initialization of various stuff needed to get the application up and
@@ -47,7 +51,7 @@ class Application(QtCore.QObject):
         in case of database connection failure"""
         from camelot.view.controls.exception import model_thread_exception_message_box
         model_thread_exception_message_box( (exc_name, exc_trace) )
-    
+
     def show_splashscreen(self):
         """:return: the splash window"""
         from PyQt4 import QtGui
@@ -79,15 +83,17 @@ class Application(QtCore.QObject):
     def pre_initialization(self):
         """Method that is called before the model thread is started, while the app is still
         running single threaded.
-        
+
         The default implementation does nothing and returns None. Overwrite
         this method for custom behaviour.
-        
-        An example use of this function would be to 
+
+        An example use of this function would be to
         present the user with a dialog to select the database to use, and to store the
         result in some global variable that is later used in the settings.ENGINE function.
         """
-        pass
+        if self.application_admin.database_selection:
+            from camelot.view.database_selection import select_database
+            select_database(self.application_admin)
 
     def start_model_thread(self):
         """Launch the second thread where the model lives"""
@@ -132,7 +138,7 @@ class Application(QtCore.QObject):
         exit
         :param application: the QApplication to run"""
         import sys
-        sys.exit(application.exec_())
+        sys.exit( application.exec_() )
 
     def initialization_exception(self, exception_info):
         """This method is called whenever an exception occurs before the event
@@ -166,11 +172,11 @@ class Application(QtCore.QObject):
             #  font.setStyleStrategy(QtGui.QFont.PreferAntialias)
             #  font.setPointSize(font.pointSize()+1)
             #  app.setFont(font)
-
-            QT_MAJOR_VERSION = float('.'.join(str(QtCore.QT_VERSION_STR).split('.')[0:2]))
-            logger.debug('qt version %s, pyqt version %s' %
-                         (QtCore.QT_VERSION_STR, QtCore.PYQT_VERSION_STR))
-            logger.debug('qt major version %f' % QT_MAJOR_VERSION)
+            if hasattr( QtCore, 'QT_MAJOR_VERSION'):
+                QT_MAJOR_VERSION = float('.'.join(str(QtCore.QT_VERSION_STR).split('.')[0:2]))
+                logger.debug('qt version %s, pyqt version %s' %
+                             (QtCore.QT_VERSION_STR, QtCore.PYQT_VERSION_STR))
+                logger.debug('qt major version %f' % QT_MAJOR_VERSION)
             app.processEvents()
             import sqlalchemy, elixir
             logger.debug('sqlalchemy version %s'%sqlalchemy.__version__)
@@ -184,7 +190,6 @@ class Application(QtCore.QObject):
             app.processEvents()
             self.start_model_thread()
             app.processEvents()
-
             #
             # WEIRD, if we put this code in a method, the translations
             # don't work
@@ -260,3 +265,4 @@ if __name__ == '__main__':
     from application_admin import ApplicationAdmin
     main(ApplicationAdmin())
 
+
