@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -32,13 +32,14 @@ from camelot.view.model_thread import model_function, gui_function, post
 
 
 class QueryTableProxy(CollectionProxy):
-    """The QueryTableProxy contains a limited copy of the data in the Elixir
+    """The QueryTableProxy contains a limited copy of the data in the SQLAlchemy
     model, which is fetched from the database to be used as the model for a
     QTableView
     """
 
     def __init__(self, admin, query_getter, columns_getter,
-                 max_number_of_rows=10, edits=None):
+                 max_number_of_rows=10, edits=None,
+                 cache_collection_proxy=None):
         """@param query_getter: a model_thread function that returns a query, can be None at construction time and set later"""
         logger.debug('initialize query table')
         self._query_getter = query_getter
@@ -47,8 +48,25 @@ class QueryTableProxy(CollectionProxy):
         #database, and as such cannot be a result of the query
         self._appended_rows = []
         super(QueryTableProxy, self).__init__(admin, lambda: [],
-                                              columns_getter, max_number_of_rows=max_number_of_rows, edits=None)
+                                              columns_getter, 
+                                              max_number_of_rows=max_number_of_rows, 
+                                              edits=None,
+                                              cache_collection_proxy=cache_collection_proxy)
 
+#    def default_sort_decorator(self):
+#        """Create a function that sorts a query, by default we sort a query by the
+#        primary keys because we always want a unique ordering of the results, eg,
+#        2 consecutive results should have the same order.  This is not necessary the
+#        case when the query is not sorted, which will result to flicker on the screen.
+#        """
+#        #
+#        # _foreign_keys is for sqla pre 0.6.4
+#        # 
+#        if hasattr(property, '_foreign_keys'):
+#            class_attribute = list(property._foreign_keys)[0]
+#        else:                             
+#            class_attribute = list(property._calculated_foreign_keys)[0]         
+        
     def get_query_getter(self):
         if not self._sort_decorator or self._query_getter==None:
             return self._query_getter
@@ -59,6 +77,10 @@ class QueryTableProxy(CollectionProxy):
             
             return sorted_query_getter
     
+    def _update_unflushed_rows( self ):
+        """Does nothing since all rows returned by a query are flushed"""
+        pass
+    
     @model_function
     def _clean_appended_rows(self):
         """Remove those rows from appended rows that have been flushed"""
@@ -170,7 +192,6 @@ class QueryTableProxy(CollectionProxy):
         row, overwrite this method for specific behaviour in subclasses"""
         if not o.id:
             self._appended_rows.append(o)
-        self._rows = self._rows + 1
 
     def remove(self, o):
         if o in self._appended_rows:
@@ -200,15 +221,39 @@ class QueryTableProxy(CollectionProxy):
             offset, limit = self._offset_and_limit_rows_to_get()
             if limit:
                 columns = self.getColumns()
-                for i, obj in enumerate( self._get_collection_range(offset, limit) ):
-                    row = i + offset
+                #
+                # try to move the offset further by looking if the
+                # objects are allready in the cache.
+                #
+                # this has the advantage that we might not need a query,
+                # and more important, that objects remain at the same row
+                # while their position in the query might have been changed
+                # since the previous query.
+                #
+                rows_in_cache = 0
+                for row in range(offset, limit + 1):
                     try:
-                        previous_obj = self.edit_cache.get_entity_at_row(row)
-                        if previous_obj != obj:
-                            continue
+                        cached_obj =  self.edit_cache.get_entity_at_row(row)                        
+                        self._add_data( columns, row, cached_obj)
+                        rows_in_cache += 1
                     except KeyError:
-                        pass
-                    self._add_data(columns, i+offset, obj)
+                        break
+                #
+                # query the remaining rows
+                #
+                query_offset = offset + rows_in_cache
+                query_limit = limit - rows_in_cache
+                if query_limit > 0:
+                    for i, obj in enumerate( self._get_collection_range(query_offset, 
+                                                                        query_limit) ):
+                        row = i + query_offset
+                        try:
+                            previous_obj = self.edit_cache.get_entity_at_row(row)
+                            if previous_obj != obj:
+                                continue
+                        except KeyError:
+                            pass
+                        self._add_data(columns, row, obj)
                 rows_in_query = (self._rows - len(self._appended_rows))
                 # Verify if rows that have not yet been flushed have been 
                 # requested
@@ -220,8 +265,9 @@ class QueryTableProxy(CollectionProxy):
 
     @model_function
     def _get_object(self, row):
-        """Get the object corresponding to row"""
-        if self._rows > 0:
+        """Get the object corresponding to row.  If row is smaller than 0
+        (in case of an invalid widget mapper), None is returned as an object"""
+        if self._rows > 0 and row >= 0:
             self._clean_appended_rows()
             rows_in_query = (self._rows - len(self._appended_rows))
             if row >= rows_in_query:
@@ -243,4 +289,3 @@ class QueryTableProxy(CollectionProxy):
                     return res.limit(1).first()
                 except:
                     pass
-
