@@ -10,17 +10,20 @@ from sqlalchemy import sql
 from elixir import ColumnProperty
 
 import camelot.types
-from camelot.model import metadata, Entity, Field, ManyToOne, OneToMany, \
-                          ManyToMany, Unicode, Date, Integer, using_options
-from camelot.view.elixir_admin import EntityAdmin
+from camelot.model import metadata
+from elixir import Entity, Field, ManyToOne, OneToMany, \
+                   ManyToMany, using_options
+from camelot.admin.entity_admin import EntityAdmin
 from camelot.view.forms import Form, TabForm, WidgetOnlyForm, HBoxForm
 from camelot.view.controls import delegates
 from camelot.view.filters import ComboBoxFilter
 from camelot.core.utils import ugettext_lazy as _
 from camelot.view.art import ColorScheme
+from sqlalchemy.types import Unicode, Date, Integer
 
 __metadata__ = metadata
 
+from action import PrintMovieListAction
 #
 # Some helper functions that will be used later on
 #
@@ -36,10 +39,11 @@ def genre_choices(entity_instance):
     (('war'),('War')),
     (('thriller'),('Thriller')),
     (('family'),('Family')) ]
-  
+
 def burn_to_disk(o_getter):
     print 'burn burn burn'
-  
+
+# begin short movie definition
 class Movie(Entity):
     using_options(tablename='movies')
     title = Field(Unicode(60), required=True)
@@ -54,6 +58,7 @@ class Movie(Entity):
     tags = ManyToMany('Tag')
     genre = Field(Unicode(15))
     rating = Field(camelot.types.Rating())
+# end short movie definition
     #
     # Camelot includes custom sqlalchemy types, like Image, which stores an
     # image on disk and keeps the reference to it in the database.
@@ -65,7 +70,7 @@ class Movie(Entity):
     #
     script = Field(camelot.types.File(upload_to='script'))
     description = Field(camelot.types.RichText)
-  
+
     #
     # Using a ColumnProperty, an sql query can be assigned to a field
     #
@@ -73,9 +78,9 @@ class Movie(Entity):
     def total_visitors(self):
         return sql.select([sql.func.sum(VisitorReport.visitors)],
                                VisitorReport.movie_id==self.id)
-    
+
     #
-    # Normal python properties can be used as well, but then the 
+    # Normal python properties can be used as well, but then the
     # delegate needs be specified
     #
     @property
@@ -87,7 +92,7 @@ class Movie(Entity):
         from camelot.container.chartcontainer import BarContainer
         return BarContainer(range(len(self.visitor_reports)),
                             [vr.visitors for vr in self.visitor_reports])
-    
+
     #
     # Each Entity subclass can have a subclass of EntityAdmin as
     # its inner class.  The EntityAdmin class defines how the Entity
@@ -101,11 +106,14 @@ class Movie(Entity):
         # the list_display attribute specifies which entity attributes should
         # be visible in the table view
         list_display = ['cover', 'title', 'releasedate', 'rating',]
+        lines_per_row = 5
         # define filters to be available in the table view
         list_filter = ['genre', ComboBoxFilter('director.full_name')]
         # if the search function needs to look in related object attributes,
         # those should be specified within list_search
         list_search = ['director.full_name']
+        # the action buttons that should be available in the list view
+        list_actions = [PrintMovieListAction('Report')]
         # the form_display attribute specifies which entity attributes should be
         # visible in the form view
         form_display = TabForm([
@@ -118,10 +126,10 @@ class Movie(Entity):
             'genre',
             'description',], columns = 2)),
           ('Cast', WidgetOnlyForm('cast')),
-          ('Visitors', WidgetOnlyForm('visitors_chart')),
+         # ('Visitors', WidgetOnlyForm('visitors_chart')),
           ('Tags', WidgetOnlyForm('tags'))
         ])
-    
+
         # create a list of actions available for the user on the form view
         # those actions will be executed within the model thread
         #
@@ -141,55 +149,49 @@ class Movie(Entity):
                                                           <tr><td>4 stars</td><td>Very good</td></tr>
                                                           <tr><td>5 stars</td><td>Awesome !</td></tr>
                                                        </table>'''),
-                                smiley=dict(delegate=delegates.SmileyDelegate))
-        #
-        # through subclassing, even more details of the look and feel can
-        # be customized, in this example, we change the number of lines of
-        # text visible in each row
-        #
-        class TableView( EntityAdmin.TableView):
-    
-            class TableWidget( EntityAdmin.TableView.TableWidget ):
-                lines_per_row = 5
-    
+                                smiley=dict(delegate=delegates.SmileyDelegate),
+                                script=dict(remove_original=True))
+
     def __unicode__(self):
         return self.title or ''
-    
+
 class Cast(Entity):
     using_options(tablename='cast')
     movie = ManyToOne('Movie')
     actor = ManyToOne('Person', required=True)
     role = Field(Unicode(60))
-  
+
     class Admin(EntityAdmin):
         verbose_name = 'Actor'
         list_display = ['actor', 'role']
         form_display = ['actor', 'role']
-  
+
     def __unicode__(self):
         if self.actor:
             return self.actor.name
         return ''
-    
+
 class Tag(Entity):
     using_options(tablename='tags')
     name = Field(Unicode(60), required=True)
     movies = ManyToMany('Movie')
-  
+
     def __unicode__(self):
         return self.name
-    
+
     class Admin(EntityAdmin):
         form_size = (400,200)
         list_display = ['name']
         form_display = ['name', 'movies']
-    
+        lines_per_row = 2
+
+# begin visitor report definition
 class VisitorReport(Entity):
     using_options(tablename='visitor_report')
     movie = ManyToOne('Movie', required=True)
-    #city = ManyToOne('City', required=True)
     date = Field(Date, required=True, default=datetime.date.today)
     visitors = Field(Integer, required=True, default=0)
+# end visitor report definition
 
     class Admin(EntityAdmin):
         verbose_name = _('Visitor Report')
